#!/bin/bash
#################################################################
#################################################################
#								#
#		    RFC-NG (New Generation)			#
# By Claudio `Sekko` Panichi - Claudio.Panichi at roma2.infn.it	#
#								#
#################################################################
#################################################################


#################################################################
#################################################################
#								#
#                          IMPORTANT				#
#								#
# Edit your rfc-ng.conf and put the absolute path to  rfc-ng.sh #
# in the following line:					#
# 								#
#								#
#################################################################

RFC_DIR=/path/to/rfc-ng.sh/directory

#################################################################
###							      ###
###   	   DO NOT TOUCH ANYTHING UNDER THIS LINE!!!	      ###
###							      ###
#################################################################
#################################################################

VERSION=3.1.0

if [ ! -f $RFC_DIR/rfc-ng.sh ]; then
	echo
	echo "               ***** FATAL *****"
	echo "Edit rfc-ng.sh and put the right value in RFC_DIR!"
	echo
	exit
fi

cd $RFC_DIR

#################################################################
#################################################################
#		     Functions used by RFC			#
#################################################################
#################################################################
#
# rfc_chk_user - Check that RFC is not running as root
# rfc_chk - Check configuration and requirements
# rfc_chk_list - Check that nodes.list exists
# rfc_help - Print the "help message"
# rfc_chk_status - Check that a node is UP and accepts ssh connections
# rfc_node_update_files - Update (or init) _files_ DB for a specific node
# rfc_node_update_users - Update (or init) _userfile_ DB for a specific node
# rfc_node_update_suid - Update (or init) _suid_ DB for a specific node
# rfc_node_chk_files - Check _files_ for a specific node
# rfc_node_chk_users - Check _userfile_ for a specific node
# rfc_node_chk_suid - Check _suid_ for a specific node


# Check that RFC is not running as root!
function rfc_chk_user() {
	if [ $UID = 0 ]; then
		echo
		echo "Sorry, it seems that you're running RFC"
		echo "as root. Please, login as 'simple' user"
		echo
		exit
	fi
}


# Check configuration and requirements
function rfc_chk() {
	rfc_chk_user
	if [ ! -f ./rfc-ng.conf ]; then
		echo
		echo "Cannot find rfc-ng.conf, exiting now!"
		echo
		exit
	fi
	source ./rfc-ng.conf
	#
        if [ ! -f $AWK_BIN ]; then
                echo
                echo "Cannot find command: awk - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $CHMOD_BIN ]; then
                echo
                echo "Cannot find command: chmod - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $CHOWN_BIN ]; then
                echo
                echo "Cannot find command: chown - You must edit rfc-ng.conf"
                echo
                exit
        fi
	if [ ! -f $DATE_BIN ]; then
                echo
                echo "Cannot find command: date - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $DIFF_BIN ]; then
                echo
                echo "Cannot find command: diff - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $FIND_BIN ]; then
                echo
                echo "Cannot find command: find - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $GREP_BIN ]; then
                echo
                echo "Cannot find command: grep - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $KILL_BIN ]; then
                echo
                echo "Cannot find command: kill - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $MAIL_BIN ]; then
                echo
                echo "Cannot find command: mail - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $MORE_BIN ]; then
                echo
                echo "Cannot find command: more - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $NMAP_BIN ]; then
                echo
                echo "Cannot find command: nmap - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! $PROC ]; then
                echo
                echo "Cannot find PROC (parallel processes) - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $SSHKEYGEN_BIN ]; then
                echo
                echo "Cannot find command: ssh-keygen - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $SSH_BIN ]; then
                echo
                echo "Cannot find command: ssh - You must edit rfc-ng.conf"
                echo
                exit
        fi
        if [ ! -f $SCP_BIN ]; then
                echo
                echo "Cannot find command: scp - You must edit rfc-ng.conf"
                echo
                exit
        fi
	if [ ! -f $SSH_DIR/id_dsa ] || [ ! -f $SSH_DIR/id_dsa.pub ] ; then
                echo
                echo "Cannot find ssh keys - You must run '$0 ssh-key-gen'"
                echo
                exit
        fi
}


# Check that nodes.list exists
function rfc_chk_list() {
        if [ ! -f ./rfc-ng.conf ]; then
                echo
                echo "Cannot find rfc-ng.conf, exiting now!"
                echo
                exit
        fi
	source ./rfc-ng.conf
	if [ !  -f $NODE_LIST ]; then
		echo
		echo "FATAL - Cannot find $NODE_LIST!"
		echo
		exit
	fi
}


# Print the "help message"
function rfc_help() {
        clear
	rfc_chk_user
        echo "#######################################################"
        echo "#                     USAGE:                          #"
        echo "#                                                     #"
        echo "# ssh-key-gen             (create new ssh keys)       #"
        echo "# ssh-key-del             (delete the ssh keys)       #"
        echo "# set-perm                (set right permissions)     #"
        echo "# node-add                (add a node to RFC list)    #"
        echo "# node-chk <node>         (check <node>)              #"
        echo "# node-update <node>      (update <node> - ask)       #"
        echo "# node-force-update <node>(update <node> - force)     #"
        echo "# node-remove <node>      (remove <node> - ask)       #"
	echo "# node-force-remove <node>(remove <node> - force)     #"
        echo "# mass-chk                (check all nodes)           #"
        echo "# mass-update             (update all nodes)          #"
        echo "# mass-remove             (remove all nodes)          #"
        echo "# list-nodes              (list all RFC nodes)        #"
	echo "# run                     (check all and send report) #"
        echo "# mail [<address>]        (send a report)             #"
	echo "# chk-config              (check RFC config)          #"
	echo "# version                 (show RFC version)          #"
        echo "#                                                     #"
        echo "#######################################################"
	echo
}


# Check that a node is UP and accepts ssh connections
function rfc_chk_status () {
	source ./rfc-ng.conf
	NMAP_RESULT=$($NMAP_BIN -P 0 -p 22 $1 2> /dev/null | $GREP_BIN "22/tcp" | $GREP_BIN open )
	if [ "$NMAP_RESULT" = "" ]; then
		echo
		echo " *** WARNING - node $1 is down, unreachable or does not exist ***"
		echo
	        rm -f $TMP_DIR/$1.lock
        	exit
	fi
}


# Update (or init) _files_ DB for a specific node
function rfc_node_update_files() {
	echo
	echo "Going to update _files_ DB for $1"
	echo
        source ./rfc-ng.conf
        NODE="$1"
        source $NODE_SETTINGS_DIR/$NODE
	if [ $CHK_FILES = 1 ]; then
		touch $NODE_PIDS_DIR/$$
		#
		$SSH $REMOTE_USER@$NODE "rm -f integrit_* aide.* fsc.* fsc-db.*"
	        $SCP $NODE_CONF_DIR/$NODE-fsc.conf $REMOTE_USER@$NODE:fsc.conf
	        $SCP $NODE_BIN_DIR/$NODE-fsc.bin $REMOTE_USER@$NODE:fsc.bin
	        $SSH $REMOTE_USER@$NODE $REMOTE_CHMOD 700 fsc.bin
	        #
	        case $FSC in
	        1)	# For AFIC ($FSC=1)
        	   	if [ -f $NODE_DB_DIR/$NODE-afick-db.dir ]; then
	                	mv -f $NODE_DB_DIR/$NODE-afick-db.pag $NODE_DB_DIR/$NODE-afick-db.pag.old
		                mv -f $NODE_DB_DIR/$NODE-afick-db.dir $NODE_DB_DIR/$NODE-afick-db.dir.old
		                mv -f $NODE_DB_DIR/$NODE-afick-db.ctr $NODE_DB_DIR/$NODE-afick-db.ctr.old
        		fi
		        $SSH $REMOTE_USER@$NODE $REMOTE_SUDO ./fsc.bin -i -c fsc.conf
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc-db.ctr 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc-db.ctr 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc-db.dir 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc-db.dir 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc-db.pag 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc-db.pag 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc.log 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc.log 2> /dev/null"
		        $SCP $REMOTE_USER@$NODE:fsc-db.ctr $NODE_DB_DIR/$NODE-afick-db.ctr
		        $SCP $REMOTE_USER@$NODE:fsc-db.dir $NODE_DB_DIR/$NODE-afick-db.dir
		        $SCP $REMOTE_USER@$NODE:fsc-db.pag $NODE_DB_DIR/$NODE-afick-db.pag
		        $SSH $REMOTE_USER@$NODE "rm -f fsc.conf fsc.log fsc-db.ctr fsc-db.dir fsc-db.pag fsc.bin"
	        	;;
	        2)	# For AIDE ($FSC=2)
		        if [ -f $NODE_DB_DIR/$NODE-aide.db ]; then
		                mv -f $NODE_DB_DIR/$NODE-aide.db $NODE_DB_DIR/$NODE-aide.db.old
		        fi
		        $SSH $REMOTE_USER@$NODE $REMOTE_SUDO ./fsc.bin --config fsc.conf --init
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER aide.db 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER aide.db 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER aide.db.new 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER aide.db.new 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc.log 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc.log 2> /dev/null"
		        $SCP $REMOTE_USER@$NODE:aide.db.new $NODE_DB_DIR/$NODE-aide.db
		        $SSH $REMOTE_USER@$NODE "rm -f aide.db aide.db.new fsc.conf fsc.log fsc.bin"
		        ;;
	        3)	# For INTEGRIT ($FSC=3)
		        if [ -f $NODE_DB_DIR/$NODE-integrit.db ]; then
		                mv -f $NODE_DB_DIR/$NODE-integrit.db $NODE_DB_DIR/$NODE-integrit.db.old
		        fi
		        $SSH $REMOTE_USER@$NODE $REMOTE_SUDO ./fsc.bin -C fsc.conf -u -q
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER integrit_current.cdb 2> /dev/null"
		        $SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER integrit_current.cdb 2> /dev/null"
		        $SCP $REMOTE_USER@$NODE:integrit_current.cdb $NODE_DB_DIR/$NODE-integrit.db
		        $SSH $REMOTE_USER@$NODE rm -f integrit_current.cdb
		        ;;
	        *)	# We shouldn't be here...
		        exit
		esac
		rm -f $NODE_PIDS_DIR/$$
	fi
}


# Update (or init) _users_ DB for a specific node
function rfc_node_update_users() {
	echo
	echo "Going to update _users_ DB for $1"
	echo
        source ./rfc-ng.conf
        NODE="$1"
        source $NODE_SETTINGS_DIR/$NODE
        if [ $CHK_USERS = 1 ]; then
		touch $NODE_PIDS_DIR/$$
		#
                if [ -f $NODE_USER_DIR/$NODE-passwd ] || [ -f $NODE_USER_DIR/$NODE-group ]; then
                        rm -f $NODE_USER_DIR/$NODE-*.old
                        for i in `ls $NODE_USER_DIR/$NODE-*`
                        do
                          mv -f $i $i.old
                        done
                fi
		#
                $SCP $REMOTE_USER@$NODE:/etc/passwd $NODE_USER_DIR/$NODE-passwd
                $SCP $REMOTE_USER@$NODE:/etc/group $NODE_USER_DIR/$NODE-group
                #
                $AWK_BIN -F: '{print $1}' $NODE_USER_DIR/$NODE-passwd > $NODE_USER_DIR/$NODE-users
                $AWK_BIN -F: '{print $1}' $NODE_USER_DIR/$NODE-group > $NODE_USER_DIR/$NODE-groups
                $AWK_BIN -F: '{print "User: "$1,"\tUID: "$3}' $NODE_USER_DIR/$NODE-passwd > $NODE_USER_DIR/$NODE-users_vs_uid
                $AWK_BIN -F: '{print "User: "$1,"\tGID: "$4}' $NODE_USER_DIR/$NODE-passwd > $NODE_USER_DIR/$NODE-users_vs_gid
                $AWK_BIN -F: '{print "User: "$1,"\tShell: "$7}' $NODE_USER_DIR/$NODE-passwd > $NODE_USER_DIR/$NODE-users_vs_shell
                #
                $AWK_BIN -F: '{print $1}' $NODE_USER_DIR/$NODE-group > $NODE_USER_DIR/$NODE-groups
                $AWK_BIN -F: '{print "Group: "$1"\t","\tGID: "$3}' $NODE_USER_DIR/$NODE-group > $NODE_USER_DIR/$NODE-groups_vs_gid
                $AWK_BIN -F: '{print "Group: "$1"\t","\tMembers: "$4}' $NODE_USER_DIR/$NODE-group > $NODE_USER_DIR/$NODE-groups_vs_members
		rm -f $NODE_PIDS_DIR/$$
        fi
}


# Update (or init) _suid_ DB for a specific node
function rfc_node_update_suid() {
	echo
	echo "Going to update _suid_ DB for $1"
	echo
	source ./rfc-ng.conf
	NODE="$1"
	source $NODE_SETTINGS_DIR/$NODE
	if [ $CHK_SUID = 1 ]; then
		touch $NODE_PIDS_DIR/$$
		#
                if [ -f $NODE_SUID_DIR/$NODE-suid ] || [ -f $NODE_SUID_DIR/$NODE-sgid ]
                then
                        rm -rf $NODE_SUID_DIR/$NODE-*.old
                        for i in `ls $NODE_SUID_DIR/$NODE-*`
                        do
                          mv -f $i $i.old
                        done
                fi
                #
		$SSH $REMOTE_USER@$NODE "rm -f suid_files sgid_files exclude_dir suid.pl"
                $SCP $FSC_DIR/suid/suid.pl $REMOTE_USER@$NODE:suid.pl
                $SCP $NODE_SETTINGS_DIR/$NODE-suid-exclude_dir.conf $REMOTE_USER@$NODE:exclude_dir
                $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_PERL suid.pl
                $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER suid_files
                $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER sgid_files
                $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER suid_files
                $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER sgid_files
                $SCP $REMOTE_USER@$NODE:suid_files $NODE_SUID_DIR/$NODE-suid
                $SCP $REMOTE_USER@$NODE:sgid_files $NODE_SUID_DIR/$NODE-sgid
                $SSH $REMOTE_USER@$NODE "rm -f suid_files sgid_files exclude_dir suid.pl"
		#
		rm -f $NODE_PIDS_DIR/$$
	fi
}


# Check _files_ for a specific node
function rfc_node_chk_files() {
        echo
        echo "Going to check _files_ on $1"
        echo
        source ./rfc-ng.conf
        NODE="$1"
        source $NODE_SETTINGS_DIR/$NODE
	if [ $CHK_FILES = 1 ]; then
		touch $NODE_PIDS_DIR/$$
		#
		$SSH $REMOTE_USER@$NODE "rm -f integrit_* aide.* fsc.* fsc-db.*"
	        case $FSC in
        	1)	# AFICK
			if [ ! -f $NODE_DB_DIR/$NODE-afick-db.pag ] && [ ! -f $NODE_DB_DIR/$NODE-afick-db.dir ]; then
		                echo "" > $LOG_DIR/$NODE-fsc.log
                		echo "FATAL! Cannot find the DBs for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
		                exit
	           	elif [ ! -f $NODE_BIN_DIR/$NODE-fsc.bin ]; then
                		echo "" > $LOG_DIR/$NODE-fsc.log
                		echo "FATAL! Cannot find the FSC binary for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
                		exit
			else
		                $SCP $NODE_CONF_DIR/$NODE-fsc.conf $REMOTE_USER@$NODE:fsc.conf
		                $SCP $NODE_BIN_DIR/$NODE-fsc.bin $REMOTE_USER@$NODE:fsc.bin
		                $SCP $NODE_DB_DIR/$NODE-afick-db.ctr $REMOTE_USER@$NODE:fsc-db.ctr
		                $SCP $NODE_DB_DIR/$NODE-afick-db.pag $REMOTE_USER@$NODE:fsc-db.pag
		                $SCP $NODE_DB_DIR/$NODE-afick-db.dir $REMOTE_USER@$NODE:fsc-db.dir
		                $SSH $REMOTE_USER@$NODE $REMOTE_CHMOD 700 fsc.bin
				$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO ./fsc.bin -k -c fsc.conf > fsc.log"
			fi
			;;
        	2)	# AIDE
			if [ ! -f $NODE_DB_DIR/$NODE-aide.db ]; then
	                	echo "" > $LOG_DIR/$NODE-fsc.log
        	        	echo "FATAL! Cannot find the DBs for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
        	        	exit
			elif [ ! -f $NODE_BIN_DIR/$NODE-fsc.bin ]; then
				echo "" > $LOG_DIR/$NODE-fsc.log
	        	        echo "FATAL! Cannot find the FSC binary for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
                		exit
			else
	                	$SCP $NODE_CONF_DIR/$NODE-fsc.conf $REMOTE_USER@$NODE:fsc.conf
	        	        $SCP $NODE_BIN_DIR/$NODE-fsc.bin $REMOTE_USER@$NODE:fsc.bin
        	        	$SCP $NODE_DB_DIR/$NODE-aide.db $REMOTE_USER@$NODE:aide.db
	        	        $SSH $REMOTE_USER@$NODE $REMOTE_CHMOD 700 fsc.bin
				$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO ./fsc.bin --config fsc.conf" > /dev/null
        	   	fi
           		;;
		3)	# Integrit
			if [ ! -f $NODE_DB_DIR/$NODE-integrit.db ]; then
				echo "" > $LOG_DIR/$NODE-fsc.log
		                echo "FATAL! Cannot find the DBs for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
                		exit
			elif [ ! -f $NODE_BIN_DIR/$NODE-fsc.bin ]; then
		                echo "" > $LOG_DIR/$NODE-fsc.log
                		echo "FATAL! Cannot find the FSC binary for $NODE." >> $LOG_DIR/$NODE-fsc.log
		                rm -f $TMP_DIR/$NODE.lock
                		exit
			else
		                $SCP $NODE_CONF_DIR/$NODE-fsc.conf $REMOTE_USER@$NODE:fsc.conf
                		$SCP $NODE_BIN_DIR/$NODE-fsc.bin $REMOTE_USER@$NODE:fsc.bin
		                $SCP $NODE_DB_DIR/$NODE-integrit.db $REMOTE_USER@$NODE:integrit_known.cdb
		                $SSH $REMOTE_USER@$NODE $REMOTE_CHMOD 700 fsc.bin
				$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO ./fsc.bin -C fsc.conf -c -q > fsc.log"
			fi
			;;
	        *) 	# Unexpected error
			echo "" > $LOG_DIR/$NODE-fsc.log
			echo "FATAL! Unexpected error on $NODE" >> $LOG_DIR/$NODE-fsc.log
			rm -f $TMP_DIR/$NODE.lock
			rm -f $NODE_PIDS_DIR/$$
			exit
	        esac
		#
        	# Rotate old log (if exists)
	        if [ -f $LOG_DIR/$NODE-fsc.log ]; then
                	mv -f $LOG_DIR/$NODE-fsc.log $LOG_DIR/$NODE-fsc.log.old
        	fi
	        #
        	# Set right permissions, move log here and then remove all files on $NODE
	        $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER fsc.log
	        $SSH $REMOTE_USER@$NODE $REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER fsc.log
	        $SCP $REMOTE_USER@$NODE:fsc.log $LOG_DIR/$NODE-fsc.log
	        $SSH $REMOTE_USER@$NODE "rm -f integrit_known.cdb aide.* fsc.* fsc-db.*"
		#
		rm -f $NODE_PIDS_DIR/$$
	fi
}


# Check _users_ for a specific node
function rfc_node_chk_users() {
        echo
        echo "Going to check _users_ on $1"
        echo
        source ./rfc-ng.conf
        NODE="$1"
        source $NODE_SETTINGS_DIR/$NODE
	if [ $CHK_USERS = 1 ]; then
		if [ ! -f $NODE_USER_DIR/$NODE-users ]; then
	        	echo "FATAL! Cannot find the userfile for $NODE." > $LOG_DIR/$NODE-rfc-chk-users.log
	        	rm -f $TMP_DIR/$NODE.lock
		        exit
		else
		        if [ -f $LOG_DIR/$NODE-rfc-chk-users.log ]; then
				mv -f $LOG_DIR/$NODE-rfc-chk-users.log $LOG_DIR/$NODE-rfc-chk-users.log.old
	        	fi
		fi
		#
		touch $NODE_PIDS_DIR/$$
	        # Take /etc/passwd and /etc/group
	        $SCP $REMOTE_USER@$NODE:/etc/passwd $NODE_USER_DIR/$NODE-today-passwd
        	$SCP $REMOTE_USER@$NODE:/etc/group $NODE_USER_DIR/$NODE-today-group
	        #
	        $AWK_BIN -F: '{print $1}' $NODE_USER_DIR/$NODE-today-passwd > $NODE_USER_DIR/$NODE-today-users
        	$AWK_BIN -F: '{print "User: "$1,"\tUID: "$3}' $NODE_USER_DIR/$NODE-today-passwd > $NODE_USER_DIR/$NODE-today-users_vs_uid
	        $AWK_BIN -F: '{print "User: "$1,"\tGID: "$4}' $NODE_USER_DIR/$NODE-today-passwd > $NODE_USER_DIR/$NODE-today-users_vs_gid
        	$AWK_BIN -F: '{print "User: "$1,"\tShell: "$7}' $NODE_USER_DIR/$NODE-today-passwd > $NODE_USER_DIR/$NODE-today-users_vs_shell
        	#
	        $AWK_BIN -F: '{print $1}' $NODE_USER_DIR/$NODE-today-group > $NODE_USER_DIR/$NODE-today-groups
        	$AWK_BIN -F: '{print "Group: "$1"\t","\tGID: "$3}' $NODE_USER_DIR/$NODE-today-group > $NODE_USER_DIR/$NODE-today-groups_vs_gid
	        $AWK_BIN -F: '{print "Group: "$1"\t","\tMembers: "$4}' $NODE_USER_DIR/$NODE-today-group > $NODE_USER_DIR/$NODE-today-groups_vs_members
	        #
        	$DIFF_BIN $NODE_USER_DIR/$NODE-users $NODE_USER_DIR/$NODE-today-users > $TMP_DIR/$NODE-chk-users
	        $DIFF_BIN $NODE_USER_DIR/$NODE-users_vs_uid $NODE_USER_DIR/$NODE-today-users_vs_uid > $TMP_DIR/$NODE-chk-users_vs_uid
        	$DIFF_BIN $NODE_USER_DIR/$NODE-users_vs_gid $NODE_USER_DIR/$NODE-today-users_vs_gid > $TMP_DIR/$NODE-chk-users_vs_gid
	        $DIFF_BIN $NODE_USER_DIR/$NODE-users_vs_shell $NODE_USER_DIR/$NODE-today-users_vs_shell > $TMP_DIR/$NODE-chk-users_vs_shell
        	#
	        $DIFF_BIN $NODE_USER_DIR/$NODE-groups $NODE_USER_DIR/$NODE-today-groups > $TMP_DIR/$NODE-chk-groups
        	$DIFF_BIN $NODE_USER_DIR/$NODE-groups_vs_gid $NODE_USER_DIR/$NODE-today-groups_vs_gid > $TMP_DIR/$NODE-chk-groups_vs_gid
	        $DIFF_BIN $NODE_USER_DIR/$NODE-groups_vs_members $NODE_USER_DIR/$NODE-today-groups_vs_members > $TMP_DIR/$NODE-chk-groups_vs_members
	        #
        	# Check for added (">") or removed ("<") users
	        echo "### The following users (if any) have changed:" > $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "> ADDED" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-users >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "< REMOVED" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-users >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-users
	        #
        	# Check UIDs
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following users (if any) have changed their UID:" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "> NEW" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-users_vs_uid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "< OLD" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-users_vs_uid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-users_vs_uid
        	#
	        # Check GIDs
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following users (if any) have changed their GID:" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "> NEW" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-users_vs_gid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "< OLD" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-users_vs_gid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-users_vs_gid
	        #
        	# Check SHELLs
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following users (if any) have changed their shell:" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "> NEW" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-users_vs_shell >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "< OLD" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-users_vs_shell >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-users_vs_shell
        	#
	        # Check for added (">") or removed ("<") groups
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following groups (if any) have changed:" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "> ADDED" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-groups >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "> REMOVED" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-groups >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-groups
        	#
	        # Check GID modification in /etc/group
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following group (if any) have different identifier:" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "> NEW" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-groups_vs_gid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "< OLD" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-groups_vs_gid >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-groups_vs_gid
        	#
	        # Check MEMBERS modification in /etc/group
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "### The following groups (if any) have different members:" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "> NEW" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-groups_vs_members >> $LOG_DIR/$NODE-rfc-chk-users.log
	        echo "" >> $LOG_DIR/$NODE-rfc-chk-users.log
        	echo "< OLD" >> $LOG_DIR/$NODE-rfc-chk-users.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-groups_vs_members >> $LOG_DIR/$NODE-rfc-chk-users.log
	        rm -f $TMP_DIR/$NODE-chk-groups_vs_members
		#
		rm -f $NODE_PIDS_DIR/$$
	fi
}


# Check _suid_ for a specific node
function rfc_node_chk_suid() {
        echo
        echo "Going to check _suid_ on $1"
        echo
        source ./rfc-ng.conf
        NODE="$1"
        source $NODE_SETTINGS_DIR/$NODE
	if [ $CHK_SUID = 1 ]; then
		if [ ! -f $NODE_SUID_DIR/$NODE-suid ] || [ ! -f $NODE_SUID_DIR/$NODE-sgid ]; then
			echo >> $LOG_DIR/$NODE-rfc-chk-suid.log
			echo "FATAL! Cannot find SUID database for $NODE" >> $LOG_DIR/$NODE-rfc-chk-suid.log
			echo >> $LOG_DIR/$NODE-rfc-chk-suid.log
			rm -f $TMP_DIR/$NODE.lock
			exit
		else
			if [ -f $LOG_DIR/$NODE-rfc-chk-suid.log ]; then
	        		mv -f $LOG_DIR/$NODE-rfc-chk-suid.log $LOG_DIR/$NODE-rfc-chk-suid.log.old
			fi
		fi
		#
		touch $NODE_PIDS_DIR/$$
		# Copy remote_suid_check.sh on $NODE, run suid.pl, copy the list of suid files and remove log/script
		$SSH $REMOTE_USER@$NODE "rm -f suid_files sgid_files exclude_dir suid.pl"
		$SCP $FSC_DIR/suid/suid.pl $REMOTE_USER@$NODE:suid.pl
		$SCP $NODE_SETTINGS_DIR/$NODE-suid-exclude_dir.conf $REMOTE_USER@$NODE:exclude_dir
		$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_PERL suid.pl"
		$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER suid_files"
		$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHOWN $REMOTE_USER sgid_files"
		$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER suid_files"
		$SSH $REMOTE_USER@$NODE "$REMOTE_SUDO $REMOTE_CHGRP $REMOTE_USER sgid_files"
		$SCP $REMOTE_USER@$NODE:suid_files $NODE_SUID_DIR/$NODE-today-suid
		$SCP $REMOTE_USER@$NODE:sgid_files $NODE_SUID_DIR/$NODE-today-sgid
		$SSH $REMOTE_USER@$NODE "rm -f suid_files sgid_files exclude_dir suid.pl"
		#
		# Looking for differences!
		$DIFF_BIN $NODE_SUID_DIR/$NODE-suid $NODE_SUID_DIR/$NODE-today-suid > $TMP_DIR/$NODE-chk-suid
		$DIFF_BIN $NODE_SUID_DIR/$NODE-sgid $NODE_SUID_DIR/$NODE-today-sgid > $TMP_DIR/$NODE-chk-sgid
		#
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "### The following files (if any) have SUID changed:" > $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "> +SUID" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-suid >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "< -SUID" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-suid >> $LOG_DIR/$NODE-rfc-chk-suid.log
		rm -f $TMP_DIR/$NODE-chk-suid
		#
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "### The following files (if any) have SGID changed:" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "> +SGID" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		$GREP_BIN ">" $TMP_DIR/$NODE-chk-sgid >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		echo "< -SGID" >> $LOG_DIR/$NODE-rfc-chk-suid.log
		$GREP_BIN "<" $TMP_DIR/$NODE-chk-sgid >> $LOG_DIR/$NODE-rfc-chk-suid.log
		rm -f $TMP_DIR/$NODE-chk-sgid
		#
		rm -f $NODE_PIDS_DIR/$$
	fi
}


#################################################################
#################################################################
#                       An now... RFC!!!                        #
#################################################################
#################################################################

case "$1" in
	version)	# Show RFC version
			echo
			echo "RFC - Version $VERSION"
			echo
			;;
	ssh-key-gen)	# Build ssh-keys
			rfc_chk_user
			if [ ! -f ./rfc-ng.conf ]; then
				echo
				echo "Cannot find rfc-ng.conf, exiting now!"
				echo
				exit
			fi
			source ./rfc-ng.conf
			#
		        if [ ! -f $SSHKEYGEN_BIN ]; then
                		echo
		                echo "Cannot find command: ssh-keygen - You must edit rfc-ng.conf"
		                echo
		                exit
		        fi
			if [ -f $SSH_DIR/id_dsa ] || [ -f $SSH_DIR/id_dsa.pub ] ; then
                		echo
		                echo "ssh keys already exist - You must run '$0 ssh-key-del'"
		                echo
                		exit
		        fi
			#
			echo
			echo "******    GOING TO BUILD SSH-KEYS     ******"
			echo
			echo "Do _NOT_ insert any passphrase when prompted"
			echo "           Press ENTER to start"
			read
			echo
			$SSHKEYGEN_BIN -t dsa -f $SSH_DIR/id_dsa
			echo
			echo
			echo "Keys are in $SSH_DIR:"
			echo
			echo "       id_dsa          # is the PRIVATE key"
			echo "       id_dsa.pub      # is the PUBLIC key"
			echo
			exit
	        	;;
	ssh-key-del)	# Remove ssh-keys
                        rfc_chk_user
			#
			if [ ! -f ./rfc-ng.conf ]; then
				echo
				echo "Cannot find rfc-ng.conf, exiting now!"
				echo
				exit
			fi
                        source ./rfc-ng.conf
                        #
			echo "GOING TO _REMOVE_ SSH KEYS!"
			echo "Are you sure? (yes/no) [no]"
			read REMOVE_SSH_KEYS
			if [ ! "$REMOVE_SSH_KEYS" ]; then
			        REMOVE_SSH_KEYS="no"
			fi
			case "$REMOVE_SSH_KEYS" in
			  yes)	rm -f $SSH_DIR/id_dsa $SSH_DIR/id_dsa.pub
				;;
			  *) 	echo
				echo "Ok, _NOT_ removing ssh-keys."
				echo
			esac
			echo
			exit
			;;
	set-perm)	# Set right permissions on files and directories
			rfc_chk_user
			#
			if [ ! -f ./rfc-ng.conf ]; then
				echo
				echo "Cannot find rfc-ng.conf, exiting now!"
				echo
				exit
			fi
                        source ./rfc-ng.conf
			#
			rfc_chk
			#			
			RFC_UID=`id -u`
			RFC_GID=`id -g`
			for i in `$FIND_BIN ./`; do $CHOWN_BIN $RFC_UID:$RFC_GID $i; done
			for i in `$FIND_BIN ./ -type f`; do $CHMOD_BIN 600 $i; done
			for i in `$FIND_BIN ./ -type d`; do $CHMOD_BIN 700 $i; done
			$CHMOD_BIN 600 $SSH_DIR/id_dsa
			$CHMOD_BIN 644 $SSH_DIR/id_dsa.pub
			$CHMOD_BIN 700 $0
			echo
			exit
			;;
	list-nodes)	# Show the content of nodes.list
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
			if [ ! -f $NODE_LIST ]; then
				echo
				echo "$NODE_LIST not found - No nodes added to RFC"
				echo
				exit
			fi
			$MORE_BIN $NODE_LIST
			;;
	chk-config)	# Check that RFC configuration is good
			rfc_chk_user
			rfc_chk
			echo
			echo "RFC configuration is right!"
			echo
			;;
	node-add)	# Add a node to nodes.list, and setup the this new node
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
			clear
			#
			echo "Hostname or IP address that you want to check:"
			read NEWHOST
			#
			if [ ! "$NEWHOST" ]; then
				echo
				echo "FATAL - you have to insert the name or IP address of the new node"
				echo
				exit
			fi
			#
			if [ `$AWK_BIN '{print $1}' $NODE_LIST | grep -x $NEWHOST` ]; then
				echo
				echo "FATAL - This node is already present in my nodes.list"
				echo
				exit
			fi
			#
			# Want do we want to check?
			clear
			echo "Do you want RFC to check for _USERFILES_ modifications on $NEWHOST? (0/1)? [1]"
			echo "(require to get /etc/passwd and /etc/group from remote node, via scp/ssh)"
			read CHK_USERS
			case $CHK_USERS in
				0)	CHK_USERS=0
				;;
				*)	CHK_USERS=1
			esac
			#
			clear
			echo "Do you want RFC to check for _SUID/SGID_ modifications on $NEWHOST (0/1)? [1]"
			echo "(require to run a perl script on remote host, via scp/ssh)"
			read CHK_SUID
			case $CHK_SUID in
				0)	CHK_SUID=0
				;;
				*)	CHK_SUID=1
					echo
					echo "Where is the _perl_ interpreter on $NEWHOST?"
					echo "[/usr/bin/perl]"
					read REMOTE_PERL
					if [ ! "$REMOTE_PERL" ]; then
						REMOTE_PERL="/usr/bin/perl"
					fi
					#
					clear
					echo "You can _SKIP_ some directory from being checked, like PROC/NFS/AFS."
					echo "Add this path(s) in $NODE_SETTINGS_DIR/$NEWHOST-suid-exclude_dir.conf"
					echo
					echo
					echo "            *** You may exclude some directory now! ***"
					echo "Please enter the paths divided by blanck spaces, like /proc and /afs"
					echo "Do not put the final '/' after each path - Press ENTER when finished"
					echo
					read EXCLUDE_DIR
					#
					touch $NODE_SETTINGS_DIR/$NEWHOST-suid-exclude_dir.conf
					for i in $EXCLUDE_DIR
						do
						echo "$i/" >> $NODE_SETTINGS_DIR/$NEWHOST-suid-exclude_dir.conf
					done
			esac
			#
			clear
			echo "Do you want RFC to check for _FILES_ modifications on $NEWHOST (0/1)? [1]"
			echo "(require to run an integrety checker on remote host, via scp/ssh)"
			read CHK_FILES
			case $CHK_FILES in
				0)	CHK_FILES=0
				;;
				*)	CHK_FILES=1
					clear
					echo "*****************************************************************"
					echo "************  CHOOSE THE FSC (File System Checker)  *************"
					echo "*****************************************************************"
					echo
					echo "The following table shows the available FSC you can put on a node"
					echo
					echo "Please check carefully if the combination OS/ARC/FSC is supported"
					echo "for the NODE you're adding and then write your answer in the form"
					echo "A.B.C where:"
					echo
					echo "        A -->   1 = AFICK, 2 = AIDE ..."
					echo "        B -->   a = linux, b = NetBSD ..."
					echo "        C -->   a = x86, b = alpha ..."
					echo
					echo "you simply have to write the A.B.C that satisfy your request! :)"
					echo "(more info can be found at rfc.sourceforge.net or in "docs" DIR)"
					echo
					echo
					echo "+---------------------------------------------------------------+"
					echo "|               |               |               |               |"
					echo "|               |     x86       |     alpha     |     sparc     |"
					echo "|               |               |               |               |"
					echo "|---------------+---------------+---------------+---------------|"
					echo "|    LINUX      | 1.a.a AFICK   | 1.a.b AFCIK   | 1.a.c AFICK   |"
					echo "|               | 2.a.a AIDE    | 2.a.b AIDE    |               |"
					echo "|               | 3.a.a INTEGRIT| 3.a.b INTEGRIT|               |"
					echo "|---------------+---------------+---------------+---------------|"
					echo "|    NetBSD     | 1.b.a AFICK   | 1.b.b AFICK   | 1.b.c AFICK   |"
					echo "|               | 2.b.a AIDE    | 2.b.b AIDE    |               |"
					echo "|               | 3.b.a INTEGRIT| 3.b.b INTEGRIT|               |"
					echo "|---------------+---------------+---------------+---------------|"
					echo "|   Solaris     | 1.c.a AFICK   | 1.c.b AFICK   | 1.c.c AFICK   |"
					echo "|---------------+---------------+---------------+---------------|"
					echo "| Digital Unix  | 1.d.a AFICK   | 1.d.b AFICK   | 1.d.c AFICK   |"
					echo "|---------------+---------------+---------------+---------------|"
					echo "|             0 (zero) for other not present here!              |"
					echo "+---------------------------------------------------------------+"
					echo
					echo "What is your choice?"
					echo
					read CHOICE
					#
					case "$CHOICE" in
					# LINUX
					1.a.a)  FSC="1"
						OS_TYPE="linux"
						NODE_ARC="x86"
						;;
					1.a.b)  FSC="1"
						OS_TYPE="linux"
						NODE_ARC="alpha"
						;;
					1.a.c)  FSC="1"
						OS_TYPE="linux"
						NODE_ARC="sparc"
						;;
					2.a.a)  FSC="2"
						OS_TYPE="linux"
						NODE_ARC="x86"
						;;
					2.a.b)  FSC="2"
						OS_TYPE="linux"
						NODE_ARC="alpha"
						;;
					3.a.a)  FSC="3"
						OS_TYPE="linux"
						NODE_ARC="x86"
						;;
					3.a.b)  FSC="3"
						OS_TYPE="linux"
						NODE_ARC="alpha"
						;;
					# NETBSD
					1.b.a)  FSC="1"
						OS_TYPE="netbsd"
						NODE_ARC="x86"
						;;
					1.b.b)  FSC="1"
						OS_TYPE="netbsd"
						NODE_ARC="alpha"
						;;
					1.b.c)  FSC="1"
						OS_TYPE="netbsd"
                				NODE_ARC="sparc"
						;;
				        2.b.a)  FSC="2"
				                OS_TYPE="netbsd"
				                NODE_ARC="x86"
				                ;;
				        2.b.b)  FSC="2"
				                OS_TYPE="netbsd"
				                NODE_ARC="alpha"
				                ;;
				        3.b.a)  FSC="3"
				                OS_TYPE="netbsd"
				                NODE_ARC="x86"
				                ;;
				        3.b.b)  FSC="3"
				                OS_TYPE="netbsd"
				                NODE_ARC="alpha"
				                ;;
				        # SOLARIS
				        1.c.a)  FSC="1"
				                OS_TYPE="solaris"
				                NODE_ARC="x86"
				                ;;
				        1.c.b)  FSC="1"
				                OS_TYPE="solaris"
				                NODE_ARC="alpha"
				                ;;
				        1.c.c)  FSC="1"
				                OS_TYPE="solaris"
				                NODE_ARC="sparc"
				                ;;
					        # DIGITAL UNIX
				        1.d.a)  FSC="1"
				                OS_TYPE="digital_unix"
				                NODE_ARC="x86"
				                ;;
				        1.d.b)  FSC="1"
				                OS_TYPE="digital_unix"
				                NODE_ARC="alpha"
				                ;;
				        1.d.c)  FSC="1"
				                OS_TYPE="digital_unix"
				                NODE_ARC="sparc"
				                ;;
					# ELSE
					0)      clear
					        echo "You have  a 'custom'  combination of  Operative-System and"
					        echo "hardware architecture. It's not a problem, but in order to"
				                echo "let RFC work properly, you  MUST procede in the  following"
				                echo "way. Please read carefully!"
				                echo
				                echo " a) Take the source-code of the FileSystem-Checker you are"
				                echo "    interested in (aide or integrit) and compile it on the"
				                echo "    remote host. It's important that you perform a  STATIC"
				                echo "    compilation!"
				                echo
				                echo " b) Once you have the STATIC  executable specific for your"
				                echo "    platform (for example 'aide' compiled for freebsd on a"
				                echo "    SUN server), put it into the directory nodes/bin. It's"
				                echo "    very important that you rename it in the standard way:"
				                echo "       mv <your executable> $NEWHOST-fsc.bin"
				                echo
				                echo "You can find the source-code  in fsc/<your-fsc>/source. If"
				                echo "you select AFICK (that is a PERL script so that it doesn't"
				                echo "need be compiled) just copy afick.pl as reported in b)."
				                echo
				                echo "Which FSC (FileSystemChecker) are you going to compile/run"
				                echo "on $NEWHOST?"
				                echo
				                echo " [1] AFICK"
				                echo " [2] AIDE"
				                echo " [3] INTEGRIT"
				                echo
				                read FSC
				                  case "$FSC" in
				                        1) FSC="1"
				        	           cp $FSC_DIR/afick/conf/other.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
					                   ;;
				                        2) FSC="2"
				                           cp $FSC_DIR/aide/conf/other.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
				                           ;;
				                        3) FSC="3"
				                           cp $FSC_DIR/integrit/conf/other.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
				                           ;;
				                        *) echo
				                           echo "Fatal: Unknown or unsopported FSC choice!"
				                           echo
				                           exit
				                  esac
						;;
				        # ERROR...
				        *)      echo
				                echo "FATAL - Unknown or unsopported OS/ARC/FSC combination!"
						echo
				                exit
				        esac
					#
					echo
			                echo "I place a default config file for the  FSC you've selected"
			                echo "that is placed in ./nodes/conf/<hostname>-fsc.conf, please"
			                echo "modify and adjust it to suite your needings!"
			                echo
			                echo "Press ENTER to continue..."
			                read
					# ask for PERL, if afick has been chosen
					clear
					if [ $FSC = 1 ]; then
						echo "Where is the _perl_ interpreter on $NEWHOST?"
						echo "[/usr/bin/perl]"
						read REMOTE_PERL
						if [ ! "$REMOTE_PERL" ]; then
							REMOTE_PERL="/usr/bin/perl"
						fi
					fi
					#
			esac
			#
			# Ask other things (if we have something to check!)
			if [ $CHK_USERS = 1 ] || [ $CHK_SUID = 1 ] || [ $CHK_FILES = 1 ]; then
				# The user that will run RFC
				clear
				echo "Please insert the username that will run RFC on $NEWHOST [rfc]"
				read REMOTE_USER
				if [ ! "$REMOTE_USER" ]; then
					REMOTE_USER="rfc"
				fi
				# Where are remote chmod/chown/chgrp/sudo
				clear
				echo "Where is the _chown_ executable on $NEWHOST?"
				echo "[/bin/chown]"
				read REMOTE_CHOWN
				if [ ! "$REMOTE_CHOWN" ]; then
					REMOTE_CHOWN="/bin/chown"
				fi
				echo "Where is the _chgrp_ executable on $NEWHOST?"
				echo "[/bin/chgrp]"
				read REMOTE_CHGRP
				if [ ! "$REMOTE_CHGRP" ]; then
					REMOTE_CHGRP="/bin/chgrp"
				fi
				echo "Where is the _chmod_ executable on $NEWHOST?"
				echo "[/bin/chmod]"
				read REMOTE_CHMOD
				if [ ! "$REMOTE_CHMOD" ]; then
					REMOTE_CHMOD="/bin/chmod"
				fi
				echo "Where is the _sudo_ executable on $NEWHOST?"
				echo "[/usr/bin/sudo]"
				read REMOTE_SUDO
				if [ ! "$REMOTE_SUDO" ]; then
					REMOTE_SUDO="/usr/bin/sudo"
				fi
				# Copy the required binary and config files
				if [ $CHK_FILES != 0 ]; then
					case "$FSC" in
					1)	# Copy AFICK executable and config
						if [ $CHOICE != 0 ]; then
				                	cp $FSC_DIR/afick/exec/afick.pl $NODE_BIN_DIR/$NEWHOST-fsc.bin
				                	cp $FSC_DIR/afick/conf/$OS_TYPE.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
						fi
				                FSC_OUTDB_OPTION="-b"
				                FSC_INIT_OPTION="-i"
				                FSC_COMPARE_OPTION="-k"
				                FSC_CONFIGFILE_OPTION="-c"
				                FSC_UPDATE_OPTION="-u"
				                ;;
				        2)      # Copy AIDE binary and config
						if [ $CHOICE != 0 ]; then
			        	        	cp $FSC_DIR/aide/exec/$OS_TYPE.$NODE_ARC.bin $NODE_BIN_DIR/$NEWHOST-fsc.bin
				                	cp $FSC_DIR/aide/conf/$OS_TYPE.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
						fi
				                FSC_INIT_OPTION="--init"
				                FSC_COMPARE_OPTION="--compare"
				                FSC_CONFIGFILE_OPTION="--config"
				                FSC_UPDATE_OPTION="--update"
				                ;;
				        3)      # Copy INTEGRIT binary and config
						if [ $CHOICE != 0 ]; then
					                cp $FSC_DIR/integrit/exec/$OS_TYPE.$NODE_ARC.bin $NODE_BIN_DIR/$NEWHOST-fsc.bin
					                cp $FSC_DIR/integrit/conf/$OS_TYPE.conf $NODE_CONF_DIR/$NEWHOST-fsc.conf
						fi
				                FSC_INIT_OPTION="-u"
				                FSC_COMPARE_OPTION="-c"
				                FSC_CONFIGFILE_OPTION="-C"
				                FSC_UPDATE_OPTION="-u"
				                ;;
				        *)      echo
				                echo "FATAL - Unexpected error."
				                echo
				                exit
				        esac
				fi
				# Backup (if present) the old nodes.list
				if [ -f $NODE_LIST ]; then
					cp -f $NODE_LIST $NODE_LIST_old
				fi
				# Add $NEWHOST in nodes.list
				echo $NEWHOST >> $NODE_LIST
				# Write parameters in settings/$NEWHOST
				echo "# Generated by RFC script" > $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Settings for $NEWHOST (change only if you know what you're doing!)" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The REMOTE RFC user for $NEWHOST:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_USER=\"$REMOTE_USER\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The filesystem-checker (fsc) that will run on $NEWHOST:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# ( 1=afick ; 2=aide ; 3=integrit )" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC=\"$FSC\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The specific _init_ option:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC_INIT_OPTION=\"$FSC_INIT_OPTION\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The specific _compare_ option:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC_COMPARE_OPTION=\"$FSC_COMPARE_OPTION\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The specific _config_ option:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC_CONFIGFILE_OPTION=\"$FSC_CONFIGFILE_OPTION\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The specific _update_ option:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC_UPDATE_OPTION=\"$FSC_UPDATE_OPTION\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# The specific _DB_ option:" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export FSC_OUTDB_OPTION=\"$FSC_OUTDB_OPTION\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Where is _chmod_ on $NEWHOST?" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_CHMOD=\"$REMOTE_CHMOD\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Where is _chown_ on $NEWHOST?" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_CHOWN=\"$REMOTE_CHOWN\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "#Where is _chgrp_ on $NEWHOST?" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_CHGRP=\"$REMOTE_CHGRP\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "#Where is _sudo_ on $NEWHOST?" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_SUDO=\"$REMOTE_SUDO\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "#Where is _perl_ on $NEWHOST?" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export REMOTE_PERL=\"$REMOTE_PERL\"" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Check for USERFILE modifications? (1=yes, 0=no)" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export CHK_USERS=$CHK_USERS" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Check for FILES modifications? (1=yes, 0=no)" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export CHK_FILES=$CHK_FILES" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "# Check for SUID/SGID modifications? (1=yes, 0=no)" >> $NODE_SETTINGS_DIR/$NEWHOST
				echo "export CHK_SUID=$CHK_SUID" >> $NODE_SETTINGS_DIR/$NEWHOST
				# Explain what to do on $NEWHOST
				clear
				echo "* Running this script, I placed some default settings for the new node *"
				echo "Please modify the .conf file to suite your needings. Be carefull: you've"
				echo "to know what you're doing; in other words, you should  read any document"
				echo "relative to the filesystem-checker you've just chosen before  you change"
				echo "the configuration file! Now I'll  give you some more infos,  please READ"
				echo "VERY CAREFULLY the following page."
				echo
				echo "Press ENTER key for more info..."
				read
				clear
				echo "Once adjusted the .conf file, you have to:"
				echo
				echo " [a] Login as root on $NEWHOST, add a USER and a GROUP called $REMOTE_USER"
				echo "     Be sure that SUDO is installed on $NEWHOST and than using visudo"
				echo "     add the following lines in /etc/sudoers.conf:"
				echo
				echo "     # <user> <hostname> = NOPASSWD:/path/to/executable <options>"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_PERL suid.pl"
				echo "     $REMOTE_USER ALL = NOPASSWD:/<PATH>/<TO>/fsc.bin"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER suid_files"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER sgid_files"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER suid_files"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER sgid_files"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER fsc-db.ctr"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER fsc-db.ctr"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER fsc-db.dir"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER fsc-db.dir"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER fsc-db.pag"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER fsc-db.pag"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER fsc.log"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER fsc.log"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER aide.db"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER aide.db"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER aide.db.new"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER aide.db.new"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHOWN $REMOTE_USER integrit_current.cdb"
				echo "     $REMOTE_USER ALL = NOPASSWD:$REMOTE_CHGRP $REMOTE_USER integrit_current.cdb"
				echo
				echo "Press ENTER to continue..."
				read
				clear
				echo " [b] copy $SSH_DIR/id_dsa.pub in the home-dir of"
				echo "     $REMOTE_USER on $NEWHOST."
				echo
				echo " [c] login as $REMOTE_USER on $NEWHOST and put id_dsa.pub in"
				echo "     ~/.ssh/known_hosts with the command"
				echo
				echo "         cat id_dsa.pub >> ~/.ssh/authorized_keys"
				echo
				echo "     Be sure that .ssh directory exist, and set permission 600 on your"
				echo "     authorized_keys file!"
				echo
				echo
				echo "Press ENTER to continue..."
				read
				clear
				echo " [d] Come back here (master node, script's dir) and try to see if ssh works"
				echo "     on $NEWHOST without password. Try to execute a command on remote node,"
				echo "     for example you can try this:"
				echo
				echo "       ssh -2 -i $SSH_DIR/id_dsa $REMOTE_USER@$NEWHOST \"ls -la /\""
				echo
				echo "     that should show the output of \"ls -la /\" on remote node."
				echo
				echo " [e] Once you're sure that ssh2 works without password, you can 'init' the"
				echo "     the database for $NEWHOST running '$0 node-update', and then the new node"
				echo "     is setted up! ;-)"
				echo
				echo "Press ENTER to continue..."
				read
				exit
			fi
			echo
			;;
	node-chk)	# Check files/suid/userfiles on remote node
                        rfc_chk_user
                        rfc_chk
                        source ./rfc-ng.conf
                        if [ ! $2 ]; then
                                echo
                                echo "Please, provide a name - Usage: $0 $1 <node-name>"
                                echo
                                exit
                        fi
                        if [ ! `$AWK_BIN '{print $1}' $NODE_LIST | $GREP_BIN -x $2` ]; then
                                echo
                                echo "FATAL - No node with this name is present in nodes.list"
                                echo
                                exit
                        fi
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/$2.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                exit
                        fi
			#
			touch $TMP_DIR/$2.lock
			rfc_chk_status "$2"
                        rfc_node_chk_files "$2"
                        rfc_node_chk_users "$2"
                        rfc_node_chk_suid "$2"
			rm -f $TMP_DIR/$2.lock
			;;
	node-remove)	# Remove a node: ask for confirmation, then call "node-force-remove"
                        rfc_chk_user
                        rfc_chk
                        rfc_chk_list
                        source ./rfc-ng.conf
                        if [ ! $2 ]; then
                                echo
                                echo "Please, provide a name - Usage: $0 $1 <node-name>"
                                echo
                                exit
                        fi
                        RM_NODE="$2"
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/*.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                exit
                        fi
			#
			if [ ! `$AWK_BIN '{print $1}' $NODE_LIST | $GREP_BIN -x $RM_NODE` ]; then
	        		echo
			        echo "FATAL - No node with this name is present in nodes.list"
			        echo
			        exit
			fi
			echo
			echo "This will remove any log, db and config file for $2. Are you sure? (yes/no) [no]"
                        read REMOVE_NODE
                        if [ "$REMOVE_NODE" = "yes" ]; then
				$0 node-force-remove $2
                        fi
			echo
                        ;;
	node-force-remove) # Remove a node, no NOT ask for confirmation
			rfc_chk_user
			rfc_chk
			rfc_chk_list
                        source ./rfc-ng.conf
			if [ ! $2 ]; then
				echo
				echo "Please, provide a name - Usage: $0 $1 <node-name>"
				echo
				exit
			fi
			RM_NODE="$2"
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/*.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                exit
                        fi
			#
			if [ ! `$AWK_BIN '{print $1}' $NODE_LIST | $GREP_BIN -x $RM_NODE` ]; then
	        		echo
			        echo "FATAL - No node with this name is present in nodes.list"
			        echo
			        exit
			fi
			touch $TMP_DIR/$2.lock
			rm -f $NODE_BIN_DIR/$RM_NODE-*
		        rm -f $NODE_USER_DIR/$RM_NODE-*
		        rm -f $NODE_SUID_DIR/$RM_NODE-*
		        rm -f $NODE_CONF_DIR/$RM_NODE-*
		        rm -f $NODE_DB_DIR/$RM_NODE-*
		        rm -f $NODE_SETTINGS_DIR/$RM_NODE-*
			rm -f $NODE_SETTINGS_DIR/$RM_NODE
		        rm -f $LOG_DIR/$RM_NODE-*
		        #
		        # Rewrite nodes.list without $RM_NODE
		        mv $NODE_LIST $NODE_LIST.tmp
		        for j in `cat $NODE_LIST.tmp`
		        do
		                if [ $j != $RM_NODE ]; then
	        	                echo $j >> $NODE_LIST
		                fi
		        done
		        rm $NODE_LIST.tmp
			touch $NODE_LIST
			echo
			echo "Done - $RM_NODE removed from RFC"
                        rm -f $TMP_DIR/$2.lock
			;;
	node-update)	# Update a node (but ask for confirmation)
                        rfc_chk_user
                        rfc_chk
                        rfc_chk_list
                        source ./rfc-ng.conf
                        if [ ! $2 ]; then
                                echo
                                echo "Please, provide a name - Usage: $0 $1 <node-name>"
                                echo
                                exit
                        fi
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/$2.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                exit
                        fi
			#
                        if [ ! `$AWK_BIN '{print $1}' $NODE_LIST | $GREP_BIN -x $2` ]; then
                                echo
                                echo "FATAL - No node with this name is present in nodes.list"
                                echo
                                exit
                        fi
                        echo
			touch $TMP_DIR/$2.lock
			rfc_chk_status "$2"
                        echo "Going to update _files_ DB for $2. Are you sure? (yes/no) [no]"
                        read UPDATE_NODE_FILES
                        if [ "$UPDATE_NODE_FILES" = "yes" ]; then
                                rfc_node_update_files "$2"
                        fi
                        echo
                        echo "Going to update _users_ DB for $2. Are you sure? (yes/no) [no]"
                        read UPDATE_NODE_USERS
                        if [ "$UPDATE_NODE_USERS" = "yes" ]; then
                                rfc_node_update_users "$2"
                        fi
                        echo
                        echo "Going to update _suid_ DB for $2. Are you sure? (yes/no) [no]"
                        read UPDATE_NODE_SUID
                        if [ "$UPDATE_NODE_SUID" = "yes" ]; then
                                rfc_node_update_suid "$2"
                        fi
			rm -f $TMP_DIR/$2.lock
                        echo
			;;
	node-force-update) # Update a node (but do not NOT ask for confirmation!)
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
                        if [ ! $2 ]; then
                                echo
                                echo "Please, provide a name - Usage: $0 $1 <node-name>"
                                echo
                                exit
                        fi
                        if [ ! `$AWK_BIN '{print $1}' $NODE_LIST | $GREP_BIN -x $2` ]; then
                                echo
                                echo "FATAL - No node with this name is present in nodes.list"
                                echo
                                exit
                        fi
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/$2.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                exit
                        fi
			#
			touch $TMP_DIR/$2.lock
			rfc_chk_status "$2"
			rfc_node_update_files "$2"
			rfc_node_update_users "$2"
			rfc_node_update_suid "$2"
                        rm -f $TMP_DIR/$2.lock
			;;
	mail)		# Send a report (via email!)
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
			if [ ! -f $NODE_LIST ]; then
				echo "RFC cannot find nodes.list!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
				exit
			fi
			#
			if [ "$2" ]; then
				MAIL_ADDRESS="$2"
			fi			
			# Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/*.lock 2> /dev/null`" ]; then
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
                                exit
                        fi
			# Put logs/*.log into global.log
			rm -f $TMP_DIR/global.log
			echo > $TMP_DIR/global.log
			echo "          *************************************************" >> $TMP_DIR/global.log
			echo "          *** Result of RFC execution on remote servers ***" >> $TMP_DIR/global.log
			echo "          *************************************************" >> $TMP_DIR/global.log
			echo >> $TMP_DIR/global.log
			echo >> $TMP_DIR/global.log
			echo >> $TMP_DIR/global.log
			echo >> $TMP_DIR/global.log
			#
			# Warn if RFC killed some processes
			if [ -f $LOG_DIR/killed.log ]; then
                                echo "*******************************************************************" >> $TMP_DIR/global.log
                                echo "*****  WARNING - RFC killed some processes because of timeout *****" >> $TMP_DIR/global.log
                                echo "*******************************************************************" >> $TMP_DIR/global.log
				echo >> $TMP_DIR/global.log
				cat $LOG_DIR/killed.log >> $TMP_DIR/global.log
				rm -f $LOG_DIR/killed.log
				echo >> $TMP_DIR/global.log
				echo >> $TMP_DIR/global.log
			fi
			#
			for i in `cat $NODE_LIST` ; do
				NODE=$i
				# Set Counter for no changes
				COUNTER=0
				source $NODE_SETTINGS_DIR/$NODE
				echo "*******************************************************************" >> $TMP_DIR/global.log
				echo "    ########  Report for $i  ########" >> $TMP_DIR/global.log
				echo "*******************************************************************" >> $TMP_DIR/global.log
			if [ $CHK_FILES = 1 ]; then
		        	if [ ! -f $LOG_DIR/$i-fsc.log ]; then
		                	echo >> $TMP_DIR/global.log
        		        	echo "*************************************" >> $TMP_DIR/global.log
	        	        	echo "*************  WARNING  *************" >> $TMP_DIR/global.log
		        	        echo "*************************************" >> $TMP_DIR/global.log
		                	echo >> $TMP_DIR/global.log
			                echo "FATAL! Cannot find logfile for $i." >> $TMP_DIR/global.log
			                echo >> $TMP_DIR/global.log
			                echo >> $TMP_DIR/global.log
		        	elif [ ! `wc -c <$LOG_DIR/$i-fsc.log` -eq 0 ]; then
						COUNTER=1
		                		echo >> $TMP_DIR/global.log
	                			echo >> $TMP_DIR/global.log
			                	echo "### The following files seem to be modified:" >> $TMP_DIR/global.log
			                	cat $LOG_DIR/$i-fsc.log >> $TMP_DIR/global.log
		        	        	echo >> $TMP_DIR/global.log
		                		echo >> $TMP_DIR/global.log
			        fi
			fi
			#
			if [ $CHK_USERS = 1 ]; then
			        if [ ! -f $LOG_DIR/$NODE-rfc-chk-users.log ]; then
			                echo "" >> $TMP_DIR/global.log
			                echo "*************************************" >> $TMP_DIR/global.log
			                echo "*************  WARNING  *************" >> $TMP_DIR/global.log
			                echo "*************************************" >> $TMP_DIR/global.log
			                echo >> $TMP_DIR/global.log
			                echo "FATAL! Cannot find userfile for $NODE." >> $TMP_DIR/global.log
			                echo "" >> $TMP_DIR/global.log
			                echo "" >> $TMP_DIR/global.log
			        elif [ ! `wc -c <$LOG_DIR/$NODE-rfc-chk-users.log` -eq 501 ]; then
						COUNTER=1
		        	       		echo "" >> $TMP_DIR/global.log
		                		cat $LOG_DIR/$NODE-rfc-chk-users.log >> $TMP_DIR/global.log
			                	echo "" >> $TMP_DIR/global.log
			                	echo "" >> $TMP_DIR/global.log
			        fi
			fi
			#
			if [ $CHK_SUID = 1 ]; then
				if [ ! -f $LOG_DIR/$NODE-rfc-chk-suid.log ]; then
			                echo "" >> $TMP_DIR/global.log
			                echo "*************************************" >> $TMP_DIR/global.log
			                echo "*************  WARNING  *************" >> $TMP_DIR/global.log
			                echo "*************************************" >> $TMP_DIR/global.log
			                echo "" >> $TMP_DIR/global.log
			                echo "FATAL! Cannot find SUID/SGID for $NODE." >> $TMP_DIR/global.log
			                echo "" >> $TMP_DIR/global.log
			                echo "" >> $TMP_DIR/global.log
			        elif [ ! `wc -c <$LOG_DIR/$NODE-rfc-chk-suid.log` -eq 140 ]; then
						COUNTER=1
			                	echo "" >> $TMP_DIR/global.log
			                	cat $LOG_DIR/$NODE-rfc-chk-suid.log >> $TMP_DIR/global.log
			                	echo "" >> $TMP_DIR/global.log
			                	echo "" >> $TMP_DIR/global.log
		        	fi
			fi
			if [ $COUNTER -eq 0 ]; then
				echo "" >> $TMP_DIR/global.log
				echo "There seems to be no changes!!!" >> $TMP_DIR/global.log
				echo "" >> $TMP_DIR/global.log
			fi
			done
			# 
			cat $TMP_DIR/global.log | $MAIL_BIN -s "$MAIL_SUBJECT" "$MAIL_ADDRESS"
			rm -f $TMP_DIR/global.log
			echo
			;;
	mass-chk)	# Check _ALL_ nodes
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
			# Warn and exit if RFC cannot find nodes.list
                        if [ ! -f $NODE_LIST ]; then
				echo "RFC cannot find nodes.list!"
                                echo "RFC cannot find nodes.list!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
                                exit
                        fi
                        # Warn and exit if one or more lock-file is present in $TMP_DIR
			if [ -n "`ls $TMP_DIR/*.lock 2> /dev/null`" ]; then
				echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
				echo "FATAL! Lock-file(s) still present in $TMP_DIR!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
                                exit
                        fi
			#
			PROC_ORIGINAL=$PROC
			# Be carefull: cannot run more jobs than hosts.
			NODES=`wc $NODE_LIST | (read n x; echo $n)`
			if [ $NODES -lt $PROC ]; then
			        PROC=$NODES
			fi
			#
			REMAINING=$NODES
			LAST=1
			RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l )`
			rm -f $NODE_PIDS_DIR/*
			rm -f $LOG_DIR/killed.log
			while [ $LAST -le $NODES ]; do
			        for n in $( seq 1 $PROC ); do
			                HOST=`head -$LAST $NODE_LIST | tail -1`
			                $0 node-chk $HOST &
			                LAST=$((LAST+1))
			        done
				sleep 2
			        RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l )`
				TIME=0
		        	while [ $RUNNING -ne 0 ]; do
		                	sleep 2
					TIME=$((TIME+2))
		                	RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l )`
					if [ $TIME -gt $TIMEOUT ]; then
						echo
						for j in `ls -C $NODE_PIDS_DIR`; do
							$KILL_BIN -SIGKILL $j
							echo "Process $j timeout: killed"
							rm -f $NODE_PIDS_DIR/$j
						done
						# Create a logfile for killed processes
						for i in `find $TMP_DIR/*.lock -type f`; do
							echo "- Removed $i, check on this host may be incomplete" >> $LOG_DIR/killed.log
						done
						rm -f $TMP_DIR/*.lock
						RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l )`
					fi
		        	done
        			REMAINING=$((NODES-LAST+1))
		        	if [ $REMAINING -lt $PROC ]; then
		                	PROC=$REMAINING
		        	fi
			done
			# be sure that all processes are not running anymore!
			while [ $RUNNING -ne 0 ]; do
			        sleep 2
			        RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l )`
			done
			;;
	mass-update)	# Update _ALL_ nodes
			rfc_chk_user
			rfc_chk
                        source ./rfc-ng.conf
                        # Warn and exit if RFC cannot find nodes.list
                        if [ ! -f $NODE_LIST ]; then
				echo "RFC cannot find nodes.list!"
                                echo "RFC cannot find nodes.list!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
                                exit
                        fi
                        # Warn and exit if one or more lock-file is present in $TMP_DIR
                        if [ -n "`ls $TMP_DIR/*.lock 2> /dev/null`" ]; then
				echo "FATAL! Lock-file(s) still present in $TMP_DIR!"
                                echo "FATAL! Lock-file(s) still present in $TMP_DIR!" | $MAIL_BIN -s "RFC WARNING" "$MAIL_ADDRESS"
                                exit
                        fi
                        #
                        PROC_ORIGINAL=$PROC
                        # Be carefull: cannot run more jobs than hosts.
                        NODES=`wc $NODE_LIST | (read n x; echo $n)`
                        if [ $NODES -lt $PROC ]; then
                                PROC=$NODES
                        fi
                        #
                        # Check for file-modification
                        REMAINING=$NODES
                        LAST=1
                        RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l 2> /dev/null )`
			rm -f $NODE_PIDS_DIR/*
			rm -f $LOG_DIR/killed.log
			rm -f $TMP_DIR/global.log
			#
                        while [ $LAST -le $NODES ]; do
                                for n in $( seq 1 $PROC ); do
                                        HOST=`head -$LAST $NODE_LIST | tail -1`
                                        $0 node-force-update $HOST 2> /dev/null &
                                        LAST=$((LAST+1))
                                done
				sleep 2
                                RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l 2> /dev/null )`
				TIME=0
                                while [ $RUNNING -ne 0 ]; do
                                        sleep 2
					TIME=$((TIME+2))
                                        RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l 2> /dev/null )`
                                        if [ $TIME -gt $TIMEOUT ]; then
                                                echo
                                                for j in `ls -C $NODE_PIDS_DIR`; do
                                                        echo "Timeout! Process $j killed"
							$KILL_BIN -SIGKILL $j
                                                        rm -f $NODE_PIDS_DIR/$j
                                                done
						# Create a logfile for killed processes
						for i in `find $TMP_DIR/*.lock -type f 2> /dev/null`; do
							echo "Removed $i, update on this host may be incomplete" >> $LOG_DIR/killed.log
						done
						rm -f $TMP_DIR/*.lock
						RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l 2> /dev/null )`
                                        fi
                                done
                                REMAINING=$((NODES-LAST+1))
                                if [ $REMAINING -lt $PROC ]; then
                                        PROC=$REMAINING
                                fi
                        done
                        # be sure that all processes are not running anymore!
                        while [ $RUNNING -ne 0 ]; do
                                sleep 2
                                RUNNING=`(ls -l $TMP_DIR/*.lock 2> /dev/null |wc -l 2> /dev/null )` 
                        done
			#
			# Warn if RFC killed some processes
                        if [ -f $LOG_DIR/killed.log ]; then
                                echo "*******************************************************************" > $TMP_DIR/global.log
                                echo "*****  WARNING - RFC killed some processes because of timeout *****" >> $TMP_DIR/global.log
                                echo "*******************************************************************" >> $TMP_DIR/global.log
                                echo "" >> $TMP_DIR/global.log
                                cat $LOG_DIR/killed.log >> $TMP_DIR/global.log
                                echo "" >> $TMP_DIR/global.log
                                echo "" >> $TMP_DIR/global.log
				cat $TMP_DIR/global.log 2> /dev/null | $MAIL_BIN -s "$MAIL_SUBJECT" "$MAIL_ADDRESS"
				rm -f $TMP_DIR/global.log
				rm -f $LOG_DIR/killed.log
                        fi
			;;
	mass-remove)	# Remove _ALL_ nodes (databases, logs and other files)
			rfc_chk_user
			rfc_chk
			echo
			echo "Going to remove logs, infos, settings, databases and temp file for _ALL_ nodes."
			echo "Are you sure (yes/no)? [no]"
			read REMOVE_FILES
			#
			if [ "$REMOVE_FILES" = "yes" ]; then
	                        source ./rfc-ng.conf
				rm -f $NODE_BIN_DIR/*
				rm -f $NODE_CONF_DIR/*
				rm -f $NODE_DB_DIR/*
				rm -f $NODE_LIST && touch $NODE_LIST
				rm -f $NODE_SETTINGS_DIR/*
				rm -f $NODE_SUID_DIR/*
				rm -f $NODE_USER_DIR/*
				rm -f $LOG_DIR/*
				rm -f $TMP_DIR/*
				echo
				echo "Done - ALL nodes just removed."
				echo
			fi
			;;
	run)		# Run a mass-check, then send the report
			$0 mass-chk
			$0 mail
			;;
  	*)		rfc_help
esac

#################################################################

exit 0
